<?php
/**
 * @file
 * Enables modules and site configuration for a minimal site installation.
 * @todo rename the sticky tag to hide the page from logged in users
 */

use Drupal\Core\Render\Element;
use Drupal\Core\Url;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\user\Entity\User;
use Drupal\cforge\Entity\Neighbourhood;

//roles
const RID_TRADER = 'trader';
const RID_COMMITTEE = 'committee';

/*
 * implements hook_form_FORM_ID_alter().
 */
function cforge_form_user_login_form_alter(&$form, $form_state) {
  $currentDest = \Drupal::request()->query->get('destination');
  drupal_set_message($currentDest, 'warning');
  if ($currentDest == 'user.login') {
    drupal_set_message(\Drupal::routeMatch()->getRouteName(), 'warning');
    if (strpos(\Drupal::routeMatch()->getRouteName(), 'user') !== FALSE) {
      $form_state->setRedirect(Url::fromUri('internal:/news'));
    }
  }
}

/*
 * implements hook_block_view_BASE_BLOCK_ID_alter().
 * Change the title of the user block
 * 
 * @param array $build
 * @param \Drupal\Core\Block\BlockPluginInterface $block
 */
function cforge_block_view_system_menu_block_alter(&$build, $block) {
  if ($block->getPluginId() =='system_menu_block:account') {
    $build['#title'] = \Drupal::currentUser()->getUsername();
    //in d8beta12, the above has no effect. This fixes it
    //$build['#configuration']['label'] = $build['#title'];
  }
}

/*
 * implements hook_cron
 */
function cforge_cron() {
  //delete old sessions to keep the sessions table small
  db_delete('sessions')
    ->condition('uid', 0)
    ->condition('timestamp', strtotime('-1 day'), '<')
    ->execute();
  db_delete('sessions')
    ->condition('timestamp', strtotime('-1 month'), '<')
    ->execute();
  //keep the watchdog table small too
  if (\Drupal::moduleHandler()->moduleExists('watchdog')) {
    db_delete('watchdog')
      ->condition('timestamp', strtotime('-1 week'), '<')
      ->execute();
  }
}

/**
 * Implement hook_form_alter().
 */
function cforge_form_alter(&$form, $form_state, $form_id) {
  switch ($form_id) {
    //add a login validation function to allow email login.
    //saves installing and configuring logintoboggan
    case 'user_login_form':
    case 'user_login_block':
    case 'user_pass':
      $login = \Drupal::config('cforge.settings')->get('login');
      if (!empty($login['email'])) {
        // This validation must happen before the main form-level validation.
        //@todo consider a $form_state->loadInclude to put this in another file
        $form['name']['#element_validate'][] = 'cforge_login_name_element_validate';
      }
      break;
  }
}

/**
 * login form element validation callback
 */
function cforge_login_name_element_validate(&$element, $form_state) {
  $properties = [
    'mail' => $form_state->getValue('name')
  ];
  $accounts = \Drupal::entityManager()->getStorage('user')->loadByProperties($properties);
  if ($account = reset($accounts)) {
    $form_state->setValue('name', $account->getUsername());
  }
  //all we've done is look up the real user name and replaced it ready for the form-level validation
}

/**
 * Implements hook_element_info_alter().
 * this is all the code from the disablepwstrength module
 * //@todo do we want to disable password strength?
 */
function cforge_element_info_alter(&$types) {
  $processes = &$types['password_confirm']['#process'];
  if (isset($processes)) {
    $position = array_search('user_form_process_password_confirm', $processes);
    if ($position !== FALSE) {
      unset($processes[$position]);
    }
  }
}

/**
 * implements hook_form_comment_form_alter().
 */
function cforge_form_comment_form_alter(&$form, &$form_state) {
  //$form['comment_body']['widget']['#title'] = t('Comment');
  //$form['author']['#access'] = FALSE;
  debug('cforge_form_comment_form_alter');
}


/**
 * implements hook_entity_info_alter().
 * add new Form handlers for the user profile edit forms
 * @see https://www.drupal.org/node/2527318
 */
function cforge_entity_type_alter(array &$entity_types) {
  if (isset($entity_types['user'])) {
    $entity_types['user']->setFormClass('profile', "Drupal\user\ProfileForm");
    $entity_types['user']->setFormClass('admin', "Drupal\user\ProfileForm");
  }
}


/**
 * Implements hook_entity_base_field_info_alter().
 * Clarify the purpose of the username
 * reduce some long text fields to string fields
 */
function cforge_entity_base_field_info_alter(&$fields, EntityTypeInterface $entity_type) {
  if ($entity_type->id() === 'user') {
    $description = t('Please use your real name. ');
    $description = ' '.t('You can log in with this.');
    $fields['name']->setDescription($description);
  }
  if ($entity_type == 'taxonomy_term') {
    $base_field_definitions['description'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Description'))
      ->setSettings(array('default_value' => '', 'max_length' => 255));
  }
}

/**
 * implements hook_user_format_name_alter().
 * append the member code to the username in brackets, if it exists.
 * for anonymous users, change the username to just the code.
 */
function cforge_user_format_name_alter(&$name, $account) {
  $uid = $account->id();
  if (\Drupal::currentUser()->isAnonymous()) {
    $name = t('Member @id', ['@id' => $uid]);
  }
  else {
    return;//@todo remove extraneous
    static $names = [];
    if (!isset($names[$uid])) {
      $avatar = \Drupal::config('cforge.settings')->get('avatar');
      $account = User::load($account->id());//@todo this shouldn't be necessary
      if (strpos($avatar, '[code]')) { 
        $names[$uid] = str_replace(
          ['[real_name]', '[code]'], 
          [$name, $account->member_code->value],
          $avatar
        );
      }
      else $names[$uid] = $name;
    }
    $name = $names[$uid];
  }
}

/**
 * implements hook_ENTITY_TYPE_view_alter().
 * remove the responsibility field if it is empty
 */
function cforge_user_view_alter(&$build, $account, $display) {
  if ($account->responsibility->isEmpty()) {
    unset($build['responsibility']);
  }
  if ($account->notes->isEmpty()) {
    unset($build['responsibility']);
  }
}

/*
 * implements hook_user_presave()
 */
function cforge_user_presave($account) {
  //if the user is being created, add the trader role
  if ($account->isNew()) {
    $account->addRole(RID_TRADER);
  }
}

/*
 * implements hook_form_FORM_ID_alter().
 */
function cforge_form_user_form_alter(&$form, $form_state) {
  if (isset($form['account'])) {
    $form['account']['status']['#options'][0] = t('User has not been approved or is no longer a member');
    $form['account']['status']['#options'][1] = t('User can log in normally');
    $form['account']['roles']['#options'][RID_COMMITTEE] .= ' - '.t('Can manage members, articles, pages images, & documents.');
    $form['account']['roles']['#options'][RID_TRADER] = ' - '.t('Can use the main currency.');
  }
  //remove the mutiple field theme from the phone numbers
  $form['phones']['widget'][0]['_weight']['#access'] = FALSE;
  $form['phones']['widget'][1]['_weight']['#access'] = FALSE;
  unset($form['phones']['widget']['#theme']);

  if (isset($form['picture'])) {
    $form['picture']['picture_upload']['#title'] .= ' ('.t('No spaces in filenames!').')';
  }
  //container for settings provided 
  if ($info = $form_state->getFormObject()->getFormDisplay($form_state)->getComponent('contact')) {
    $form['contact']['#title'] = t('Receiving mail from the site');
  }
}

/**
 * implements hook field_widget_WIDGET_NAME_form_alter().
 * tailor the display of the phone multiple cardinality field
 */
function cforge_field_widget_telephone_default_form_alter(&$element, $form_state, $context) {
  static $i = 0;
  $element['value']['#field_prefix']  = ($i%2 == 1) ? t('Other phone:') : t('Mobile phone:');
  $i++;
}
/**
 * implements hook field_widget_WIDGET_NAME_form_alter().
 * reformat the address widget assuming there is no choice of country and neighbourhood is a dropdown
 * restrict the available countries to those in config
 */
function cforge_field_widget_address_default_form_alter(&$element, $form_state, $context) {
  $element['country_code']['#type'] = 'hidden';
  $element['dependent_locality']['#options'] = cforge_list_neighbourhoods();
  $element['dependent_locality']['#type'] = 'select';
  unset($element['dependent_locality']['#size']);
}

/*
 * implements hook_form_FORM_ID_alter().
 * tailor the account settings form
 */
function cforge_form_user_admin_settings_alter(&$form) {
  $reg_fieldset = &$form['registration_cancellation'];
  unset(
    $reg_fieldset['#description'], 
    $reg_fieldset['user_register']['#options'][USER_REGISTER_VISITORS]
  );
  $reg_fieldset['#weight'] = -5;
  $reg_fieldset['user_register']['#title'] = t('How to add new members');
  $reg_fieldset['user_register']['#options'][USER_REGISTER_ADMINISTRATORS_ONLY] = t('Only committee can create accounts');
  $reg_fieldset['user_register']['#options'][USER_REGISTER_VISITORS_ADMINISTRATIVE_APPROVAL] = t('Anyone can create an account, then a committee member must enable it');
  $reg_fieldset['user_cancel_method']['#access'] = FALSE;
  $reg_fieldset['user_email_verification']['#access'] = FALSE;
  
  $form['contact']['#access'] = FALSE;
  $form['admin_role']['#access'] = FALSE;
  
  $reg_fieldset['account_manager'] = [
    '#title' => t('Account manager'),
    '#description' => t('Will receive an email whenever a new account is created'),
    '#type' => 'entity_autocomplete',
    '#target_type' => 'user',
    '#selection_handler' => 'default:user',
    '#tags' => FALSE,
    '#default_value' => User::load(\Drupal::Config('cforge.settings')->get('account_manager')),
    '#required' => TRUE,
  ];
  $form['#submit'][] = 'cforge_save_account_manager';
  //on a new installation there may be no members with committee role
  if (count(\Drupal::entityQuery('user')->condition('roles', [RID_COMMITTEE])->execute())) {
    $reg_fieldset['account_manager']['#selection_settings']['filter']['role'] = [RID_COMMITTEE];
  }
}

/**
 * form submit callback for user_admin_settings
 * save the referrrer for use in the mail
 */
function cforge_save_account_manager($form, $form_state) {
  \Drupal::ConfigFactory()->getEditable('cforge_currencies.settings')
    ->set('account_manager', $form_state->getValue('account_manager'))
    ->save();
}

/*
 * implements hook_form_FORM_ID_alter().
 * force users to put pages in one of the four flattened menus
 * (other node types have no menus at all)
 */
function cforge_form_node_page_form_alter(&$form, &$form_state) {
  //prevent changing the alias of the home page, and remove the delete button
  if ($form['path']['widget'][0]['alias']['#default_value'] == '/home') {
    $form['path']['widget']['#access'] = FALSE;
    $form['path']['warning'] = [
      '#markup' => t(
        "Warning: '@name' is your site's front page. It cannot be changed.",
        ['@name' => '/home']
      )
    ];
  }
  cforge_flatten_menu_widget_options($form['menu']['link']['menu_parent']);
  $form['menu']['link']['menu_parent']['#title'] = t('Put in menu');
  $form['menu']['enabled']['#description'] = t('Pages without menu items may be harder to find.');
  
  //change the meaning of the sticky flag
  unset($form['sticky']['#group']);
  $form['sticky']['widget']['value']['#title'] = t('This page is for anonymous users only (and committee)');
  
}

/**
 * implements hook_form_node_form_alter().
 * make the descriptions of the aliases more friendly
 */
function cforge_form_node_form_alter(&$form, $form_state) {
  $type = $form_state->getFormObject()->getEntity()->bundle();
  //rename the url alias field and explain it better
  $alias_fieldset = &$form['path']['widget'][0]['alias'];
  if ($type == 'image') {
    $form['path']['#access'] = FALSE;
  }
  else {
    $alias_fieldset['#title'] = t('Friendly address');
    $alias_fieldset['#description'] = t(
      "Optionally give your page a more meaningful address. E.g. 'articles-of-association'"
    );
  }
}

/*
 * implements hook_menu_links_discovered_alter().
 * remove unwanted menu items put in the main menu by core modules
 */
function cforge_menu_links_discovered_alter(&$links) {
  unset(
    $links['node.add_page'], //poor default in the node module
    $links['filter.tips_all'], //was disabled anyway
    $links['search.view'], //was disabled anyway
    //@see Drupal\cforge\Routing\RouteSubscriber
    $links['entity.menu.add_form'], //created by admin_toolbar_tools module
    $links['masquerade.unmasquerade']//access bug in beta of masquerade module
  );
  $links['contact.site_page']['enabled'] = TRUE;
  //having removed node.add_page its is helpful to rename the links under it
  foreach (\Drupal::entityManager()->getStorage('node_type')->loadMultiple() as $type) {
    $type_id = $type->id();
    if (isset($links['node.add.'.$type_id])) {
      $links['node.add.'.$type_id]['title'] = t(
        'Add @type',
        ['@type' => $links['node.add.'.$type_id]['title']]
      );
    }
  }
  //but instead we want a link to each vocabulary's overview form in the tools menu
  $vocabs = \Drupal::entityManager()
    ->getStorage('taxonomy_vocabulary')
    ->loadMultiple();
  foreach ($vocabs as $vid => $vocab) {
    $links['entity.taxonomy_vocabulary.overview_form.cforge.'.$vid] = [
      'title' => t('Manage @name', ['@name' => $vocab->label()]),
      'description' => $vocab->getDescription(),
      'route_name' => 'entity.taxonomy_vocabulary.overview_form',
      'route_parameters' => ['taxonomy_vocabulary' => $vid],
      'menu_name' => 'tools'
    ];
    //The admin_toolbar_tools module puts its own links...
    $key = 'entity.taxonomy_term.add_form.'. $vid;
    if (isset($links[$key])) {
      unset($links[$key]);
    }
  }
}

/**
 * implements hook_menu_local_actions_alter().
 * we need to remove both the route AND the menu links, actions and task that refer to it
 */
function cforge_menu_local_actions_alter(&$items) {
  //@see Drupal\cforge\Routing\RouteSubscriber
  unset($items['entity.menu.add_form']);
}

/*
 * implements hook_form_FORM_ID_alter().
 * Ensure that the user-visible menus are all flat
 */
function cforge_form_menu_link_edit_alter(&$form, $form_state) {
  cforge_flatten_menu_widget_options($form['menu_parent']);
  //prevent normal users from putting links in unusual menus
  if (\Drupal::currentUser()->id > 1) {
    unset(
      $form['menu_parent']['admin:'], 
      $form['menu_parent']['tools:'], 
      $form['menu_parent']['devel:']//just in case
    );
  }
}

/*
 * implements hook_form_FORM_ID_alter().
 * prevent menu items being dragged into a hierarchy
 * if settings flag is checked
 */
function cforge_form_menu_edit_form_alter(&$form, $form_state) {
  $menu_name = $form_state->getFormObject()->getEntity()->id();
  if (in_array($menu_name, ['main', 'account', 'footer'])) {
    if (\Drupal::config('cforge.settings')->get('flat_menus')) {
      foreach (Element::Children($form['links']['links']) as $key) {
        $form['links']['links'][$key]['#attributes']['class'][] = 'tabledrag-root';
      }
    }
  }
}

/**
 * utility function
 * flatten the menu link options to ensure links are only placed at the top level
 * @param array $element
 *   a select form element whose options are all the menu links in all the menus
 * @return array
 *   the same $element but with only menu names in it.
 */
function cforge_flatten_menu_widget_options(&$element) {
  //flatten the menu, so you can only put things at the top level
  foreach ($element['#options'] as $key => $linkname) {
    if (substr($key, -1) != ':') {
      unset($element['#options'][$key]);
    }
  }
}

/*
 * implements hook_mail_alter().
 * just some tweaks to various mails
 */
function cforge_mail_alter(&$message) {
  $key = $message['id'];
  //remove the unsubscribe message because subscribing is necessary!
  if ($key == 'user_mail') {
    //don't encourage users to change their contact settings
    unset($message['body'][count($message['body'])-3]);
  }
  elseif($key == 'register_pending_approval_admin') {
    //divert it to the accounts manager
    if ($uid = \Drupal::Config('cforge.settings')->get('account_manager')) {
      $account = user_load($uid);
      $message['to'] = $account->getEmail();
      $message['params']['user'] = $account;
    }
  }
  
  $recipient_accounts = \Drupal::entityManager()
    ->getStorage('user')
    ->loadByProperties(['mail' => $message['to']]);
  if ($recipient_accounts) {
    //add the mail footer
    $message['body'][] =  \Drupal::token()->replace(
      \Drupal::config('cforge.settings')->get('mail_footer'),
      $message['params'],
      [
        'language' => $message['langcode'], 
        'sanitize' => FALSE
      ]
    );
  }
  
  //@todo remove this test
  drupal_set_message('mail was logged, not sent');
  $message['send'] = 0;
  $string = $message['to'].'<br />';
  $string.= $message['subject'].'<br />';
  $string.= implode("\n", $message['body']).'<br />';
  
  Drupal::logger('cforge')->info($string);
  
}

/*
 * menu callback
 * shortcut to the backup & restore module download backup
 * @todo test when backup_migrate is available
 */
function backup_direct() {
  //@todo flush all caches
  $form_state['values'] = [
    'profile_id' => NULL,
    'destination_id' => 'download',
    'source_id' => NULL,
  ];
  drupal_form_submit('backup_migrate_ui_manual_quick_backup_form', $form_state);
  drupal_goto(BACKUP_MIGRATE_MENU_PATH .'/restore');
}

/**
 * Implements hook_form_system_modules_alter().
 * disable the cforge_hosted checkbox, if present
 * Add Readme links to the modules page, where readme files exist.
 */
function cforge_form_system_modules_alter(&$form, &$form_state) {
  //add a link to the readme file, similar to the advanced help module
  foreach (Element::children($form['modules']) as $group) {
    foreach (Element::children($form['modules'][$group]) as $module) {
      $path = DRUPAL_ROOT .'/'.drupal_get_path('module', $module);
      if (file_exists($path.'/README.txt')) {
        $form['modules'][$group][$module]['links']['help'] = [
          '#type' => 'link',
          '#title' => t('README.txt'),
          '#url' => Url::fromRoute('cforge.readme', ['module' => $module]),
          '#attributes' => [
            'class' => ['module-link']
          ]
        ];
      }
    }
  }
  if (\Drupal::currentUser()->id() > 1) {
    $other = t('Other');
    if (array_key_exists('cforge_hosted', $form['modules'][$other])) {
      $form['modules'][$other]['cforge_hosted']['enable']['#disabled'] = TRUE;
      $form['modules'][$other]['cforge_hosted']['description']['#markup'] .= strtoupper(t('Only user 1 can disable this module'));
    }
  }
}

/**
 * implements hook_system_info_alter().
 * @param array $info 
 * @param Extension $extension
 * has $name $type, $description, $package
 */
function cforge_system_info_alter(array &$info, &$extension) {
  //make some contrib modules into cforge features!
  switch ($extension->getName()) {
    case 'community_tasks':
      $info['package'] = (string)t('Hamlets features');
  }
}

/**
 * implements hook_form_alter().
 * clears the css cache after a modification
 * @todo test when css_injector is available
 */
function cforge_form_css_injector_edit_alter(&$form, &$form_state) {
  $form['buttons']['save']['#submit'][] = 'drupal_clear_css_cache';
  $form['buttons']['save_and_continue']['#submit'][] = 'drupal_clear_css_cache';
}

/**
 * utility function to make batch mail a little easier for cforge dependencies
 * 
 * @param string $module
 * @param string $key
 * @param array $uids
 * @param array $params
 * @param \Drupal\Core\Url|string $redirect
 * @param string $finished
 *   the path to go to at the end of the batch
 * 
 * @todo test batch mail in d8
 */
function cforge_batch_mail($module, $key, $uids, $params, $redirect, $finished = '') {
  if (empty($uids)) return;
  $batch = [
    'title' => t('Sending @count mails', ['@count' => count($uids)]),
    //@see https://api.drupal.org/api/drupal/core!includes!form.inc/function/batch_process/8
    'batch_redirect' => $redirect
  ];
  if ($finished) {
    $batch['finished'] = $finished;
  }
  foreach ($uids as $uid) {
    $batch['operations'][] = [
      'cforge_batch_mail_send',
      [$module, $key, $uid, $params]
    ];
  }
  batch_set($batch);
}

/**
 * batch callback
 */
function cforge_batch_mail_send($module, $key, $uid, $params, $progress) {
  $account = user_load($uid);
  $params['user'] = $account;
  \Drupal::service('plugin.manager.mail')->mail(
    $module, 
    $key,
    $account->getEmail(), 
    $account->getPreferredLangcode(), 
    $params
  );
}

/**
 * implements hook_node_access().
 * Hamlets changes the meaning of the sticky flag
 * A sticky flag now means the node cannot be seen by authenticated users
 * @todo best not to hack the sticky flag but use a new form widget and a keyValue pair
 */
function cforge_node_access($node, $op, $account) {
  if ($node->isSticky() and $node->getEntityTypeId() == 'page') {
    if ($account->isAnonymous()) {// or $account->hasPermission('administer nodes')
      return AccessResult::allowed()->cachePerPermissions();
    }
    else return AccessResult::forbidden();
  }
  return AccessResult::neutral();
}


/**
 * implements hook_entity_extra_field_info().
 * puts a details field used by the broadcast module and the latest module
 */
function cforge_entity_extra_field_info() {
  $handler = \Drupal::moduleHandler();
  $fields = [];
  if ($handler->moduleExists('cforge_latest') || $handler->moduleExists('cforge_broadcast')) {
    $fields['user']['user']['form']['notifications'] = [
      'label' => t('Mail notification settings'),
      'description' => t('If applicable'),
      'weight' => 8,
      'visible' => TRUE
    ];
  }
  return $fields;
}

/**
 * implements hook_views_data_alter().
 * populate the neighbourhoods views exposed filter
 * 
 * @todo should this go in a views.inc file?
 */
function __cforge_views_data_alter(&$tables) {
  $tables['user__address']['address_dependent_locality']['filter']['options callback'] = 'cforge_list_neighbourhoods';
}

/**
 * utility function
 * make a list of of all the neighbourhood entities
 * @return Neighbourhood[]
 *   both keys and values are the entity label, order alphabeticaly
 */
function cforge_list_neighbourhoods() {
  $options = [];
  foreach (Neighbourhood::loadMultiple() as $neighbourhood) {
    $options[] =  $neighbourhood->label();
  }
  sort($options);
  return array_combine($options, $options);
}

